//Fractal Landscape

//Compiler Variables
#define FullCpu 1 //Use Windows Events (0) [Only update on a message event return) or DirectX Events (1) [Immediate frame updates]
#define UseOcclusion 0 //Use occlusion culling
#define ScreenWidth 800
#define ScreenHeight 600
#define MaxFracLevs 5
#define MinFracLevs 3 //Do not change this to lower than 3 because the code to handle areas less than 3 was taken out, as I decided it wasn't really needed.
#define MoveSpeed .06f
#define AreaViewable 45 //Degress, Do not set to 180 or 360

//Includes
#include <windows.h>
#include <d3dx9.h>
#include "resource.h"

//Only include direct input if the full CPU is being used, otherwise allow windows to handle
#if FullCpu==1
	#include <dinput.h>
	#define NonFullCPU_DoFrame() //Only call DoFrame on state changes if not running Full CPU
#else
	#define NonFullCPU_DoFrame() DoFrame()
#endif

//Occlusion culling macro to only use certain lines if Occlusion Culling is turned on
#if UseOcclusion==1
	#define UO(l) l
	#define NUO(l)
	#define OrOp 0xFF000000
#else
	#define UO(l)
	#define NUO(l) l
	#define OrOp 0
#endif

//Macros
#define RandF()  (rand()/(RAND_MAX/2.0f)-1.0f) // -1.0  -  1.0
#define DisplayError(a) MessageBox(NULL, a, "Error", MB_OK);return false; //Display error message and return from function with false
inline DWORD F2DW(FLOAT f) { return *((DWORD*)&f); } //Convert DWORD to float

struct Vertex //Vertex structure for the heightmap vertecies
{
	float x,y,z; //Position
	float xn,yn,zn; //Normal
	DWORD Color;
};

struct AreaStruct //A quadtree area structure
{
	D3DXVECTOR3 PointBounds[8]; //Pointbounds of bounding box, 0 & 8 are max and min
	D3DXPLANE PlaneBounds[6]; //Planebounds of bounding box
UO(	IDirect3DVertexBuffer9 *BoxBuf; ) //Quadtree area bounding box buffer
	IDirect3DVertexBuffer9 *VertBuf; //Quadtree area vertex buffer
};

struct BubbleStruct //A bubble for when the eye is under water
{
	D3DXVECTOR3 Current;
	float Magnitude;
	int Amplitude, Speed,FrameOn;
};

bool InitDX(); //Init Direct3D and devices
void ReleaseDX(); //Releases Direct3D and devices
void MoveSun(); //Move the sun and light position
void InitFractal(); //Resets the fractal height map and rebuilds
void FractalLevelInc(); //Increases the fractal height map array by 2 areas
void FractalLevelDec(); //Decreases the fractal height map array by 2 areas
void FractalLevelMake(); //Reinterpolates current fractal level height values
void MoveFractalToVB(); //Moves the fractal height values to a vertex buffer
void MakeCloud(); //Uses the height map to create a cloud texture
void DoFrame(); //Renders a frame
void MoveCamera(); //Moves the camera when an action is performed
LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM); //Processes events
bool QuadInFrustrum(AreaStruct*); //Checks to see if a quadtree area is within the viewing frustrum
bool EdgeIntersectsFace(D3DXVECTOR3*,D3DXVECTOR3*,D3DXPLANE*); //Checks to see if an edge intersects a face, only used in the previous function
void WriteProcessing(); //Writes the word "Processing" on the screen
void ProcessKey(char,HWND); //Processes keyboard input
#if FullCpu==1
	void FPSDraw(); //Draws the fps on the screen, only needed if full cpu is being used
#endif

//Global Variables for D3D Engine
IDirect3D9 *g_pD3D = NULL;
IDirect3DDevice9 *g_pD3DDevice = NULL;
IDirect3DTexture9 *SunTex, *CloudTex=NULL, *Bubble; //Engine textures
UO(IDirect3DQuery9 *AreaQuery;)
AreaStruct *Areas=NULL; //Quadtree areas list
D3DXVECTOR3 CameraPoints[8], SunPos; //Camera frustrum points and SunPos
D3DXPLANE CameraPlanes[6]; //Camera frustrum planes
D3DPRESENT_PARAMETERS d3dpp; //Parameters for viewing screen
D3DXMATRIXA16 MatView,MatProj,SunProj; //Transformation matricies
HWND MainHwnd; //Global handle to the window
ID3DXFont *g_pFont=NULL; //Font for FPS
ID3DXSprite *BubbleSprite; //Sprite for bubbles

//Global Variables for Fractal Engine
int FractalLevel=0,RemRand=5; //Fractal level currently on and variable to allow user to go to a predetermined example
int NumberOfAreaRows; //Number of quadtree rows/columns
int SunAng=225,MousePos[2]; //Self explanatory
float MulBy=4; //Roughness on each fractal level
float *FractalPointer=NULL; //List of height values
float WorldAt[3]={20.75,20.75,0}, CameraAngle=180,CameraAngle2=0; //Where the eye/camera is and its angles on the X & Y axis
bool FogOn=true,LightOn=true,InInit=true,RemakeBuf=false,CloudOn=true; //Booleans for options
DWORD FrameOn=0; //Count of every frame in the engine, used to watch for user pressing or holding keys
BubbleStruct Bubbles[20]; //Bubbles values
float BubbleDisplacementArray[20]; //Array of bubble x displacements

int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR pCmdLine,int nCmdShow)
{
	//Creates the window
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_HREDRAW | CS_VREDRAW | CS_OWNDC, WndProc, 0L, 0L, hInst, LoadIcon(hInst, "MAIN_ICON"), NULL, NULL, NULL, "Fractal", NULL };
	if(RegisterClassEx(&wc)==0)
		{DisplayError("Cannot Create Window")}

	MainHwnd = CreateWindow("Fractal", "Fractal Landscape", WS_OVERLAPPEDWINDOW, 0, 0, ScreenWidth,ScreenHeight, NULL, NULL, hInst, NULL);
	if(MainHwnd==NULL)
		{DisplayError("Cannot Create Window")}

	if(!InitDX())
		{DisplayError("Cannot Init DX, Incompatible Graphics Card")}

	WndProc(MainHwnd, WM_CHAR, 'm',0); //Centers the mouse

	//Shows the window
	ShowWindow(MainHwnd, nCmdShow);
	UpdateWindow(MainHwnd);

	//Create bubble x displacements from sin wave
	for(int i=0;i<20;i++)
		BubbleDisplacementArray[i]=(float)sin(i*D3DX_PI/10);

	//Set up initial bubble variables if starting in water
	srand(GetTickCount());
	for(i=0;i<20;i++)
	{
		Bubbles[i].Current=D3DXVECTOR3(ScreenWidth/2-8,ScreenHeight/2-8,0); //Start in exact center of screen
		Bubbles[i].Magnitude=(RandF()+1)*30; //0 - 30
		Bubbles[i].Amplitude=rand()%6+3; //3-8
		Bubbles[i].Speed=rand()%4; //0-3
		Bubbles[i].FrameOn=Bubbles[i].Speed-i; //Start each bubble 1 after the other, starting with the first
	}

	InitFractal();

	//Main loop
	MSG msg;
	ZeroMemory(&msg, sizeof(MSG));
	#if FullCpu==0 //If not full cpu, only run as a normal windows app
		while(GetMessage(&msg, NULL, 0, 0))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	#else //If full cpu
		//Create the DX hook to the mouse and keyboard
		IDirectInput *g_pDI;
		IDirectInputDevice *g_pMouse;
		LPDIRECTINPUTDEVICE g_pKeyboard;
		DIMOUSESTATE MouseStruct;
		BYTE keys[256];
		if(FAILED(DirectInputCreate(GetModuleHandle(NULL), DIRECTINPUT_VERSION, &g_pDI, NULL)))
			{DisplayError("Cannot Instance Input Devices")}
		if(FAILED(g_pDI->CreateDevice(GUID_SysMouse, &g_pMouse, NULL)))
			{DisplayError("Cannot Instance Mouse")}
		if(FAILED(g_pDI->CreateDevice(GUID_SysKeyboard, &g_pKeyboard, NULL)))
			{DisplayError("Cannot Instance Keyboard")}
		if(FAILED(g_pMouse->SetDataFormat(&c_dfDIMouse)))
			{DisplayError("Cannot Configure Mouse")}
		if(FAILED(g_pKeyboard->SetDataFormat(&c_dfDIKeyboard)))
			{DisplayError("Cannot Configure Keyboard")}
		g_pMouse->SetCooperativeLevel(MainHwnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);
		g_pKeyboard->SetCooperativeLevel(MainHwnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);
		g_pMouse->Acquire();
		g_pKeyboard->Acquire();

		//Start the infinite program loop that ties up the processor
		while(true)
		{
			if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
			{
				if(GetMessage(&msg, NULL, 0, 0)==0)
					break;
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			//Check for mouse input and change angles accordingly
			ZeroMemory(&MouseStruct, sizeof(MouseStruct));
			if(FAILED(g_pMouse->GetDeviceState(sizeof(DIMOUSESTATE), &MouseStruct)))
				g_pMouse->Acquire();
			else if(MouseStruct.lX!=0 || MouseStruct.lY!=0)
			{
				CameraAngle+=D3DX_PI/-360*MouseStruct.lX;
				CameraAngle2+=D3DX_PI/-360*MouseStruct.lY;
				MoveCamera();
			}
			//Check for keyboard input and respond accordingly
			ZeroMemory(keys, sizeof(keys));
			if(FAILED(g_pKeyboard->GetDeviceState(sizeof(keys), keys)))
				g_pKeyboard->Acquire();
			else
				for(int i=0;i<256;i++)
					if(keys[i]&0x80)
						ProcessKey((char)i,MainHwnd);
			DoFrame(); //Draw a frame
		}
		//Release keyboard and mouse when program ends
		g_pDI->Release();
		g_pMouse->Release();
		g_pKeyboard->Release();
	#endif

	//Shutdown
	ReleaseDX();
	g_pD3D->Release();
	delete[] FractalPointer;
	UnregisterClass("Fractal", hInst);
	return true;
}

bool InitDX() //Init Direct3D and devices
{
	if(d3dpp.BackBufferWidth==0) //One time only initialization of D3D interface and display mode variables
	{
		if((g_pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL)
			{DisplayError("Cannot Init DX Instance")}

		D3DDISPLAYMODE d3ddm;
		if(FAILED(g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
			{DisplayError("Cannot Get Display Adapter")}

		d3dpp.Windowed = true;
		d3dpp.hDeviceWindow = MainHwnd;
		d3dpp.BackBufferWidth = ScreenWidth;
		d3dpp.BackBufferHeight = ScreenHeight;
		d3dpp.BackBufferCount = 1;
		d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;
		d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		d3dpp.BackBufferFormat = d3ddm.Format;
		d3dpp.EnableAutoDepthStencil = TRUE;
		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
		d3dpp.Flags = D3DPRESENTFLAG_VIDEO;
	}

	//Create the d3d device
	if(FAILED(g_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, MainHwnd, D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &g_pD3DDevice)))
		{DisplayError("Cannot Create DX Device")}
	
	//Set engine render states
	g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_USEW);
	g_pD3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	g_pD3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	g_pD3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x01);
	g_pD3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
UO(	g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true); )
UO(	g_pD3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, true); )
	g_pD3DDevice->SetRenderState(D3DRS_FOGENABLE, true);
	g_pD3DDevice->SetRenderState(D3DRS_FOGVERTEXMODE,  D3DFOG_LINEAR);
	g_pD3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_MIRROR); //Wrap cloud textures
	g_pD3DDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_MIRROR);	

	//Add lighting
	g_pD3DDevice->SetRenderState(D3DRS_AMBIENTMATERIALSOURCE, D3DMCS_COLOR1); //Sets up ambient lighting
	g_pD3DDevice->SetRenderState(D3DRS_AMBIENT,0x555555);
	g_pD3DDevice->LightEnable(0, TRUE);
	g_pD3DDevice->SetRenderState(D3DRS_SHADEMODE, D3DSHADE_PHONG);

	//Sets up world and projection matricies
	D3DXMATRIXA16 MatWorld;
	g_pD3DDevice->SetFVF(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_NORMAL);
	D3DXMatrixPerspectiveFovLH(&MatProj, D3DX_PI*AreaViewable/180, (float)ScreenWidth/ScreenHeight, 0.001f, 200.0f);
	D3DXMatrixPerspectiveFovLH(&SunProj, D3DX_PI*AreaViewable/180, (float)ScreenWidth/ScreenHeight, 0.001f, 600.0f);
	D3DXMatrixIdentity(&MatWorld);
	g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &MatProj);
	g_pD3DDevice->SetTransform(D3DTS_WORLD, &MatWorld);

	//Create sun
	HGLOBAL PicAdr=LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(SunBmp), RT_BITMAP));
	D3DXCreateTextureFromFileInMemoryEx(g_pD3DDevice, PicAdr,49206,D3DX_DEFAULT,D3DX_DEFAULT,1,0,D3DFMT_A8R8G8B8,D3DPOOL_MANAGED,D3DX_FILTER_NONE,D3DX_FILTER_NONE,D3DCOLOR_RGBA(255,0,0,0),NULL,NULL,&SunTex);
	DeleteObject(PicAdr);
	
	//Create bubbles
	PicAdr=LoadResource(NULL, FindResource(NULL, MAKEINTRESOURCE(BubblePic), RT_BITMAP));
	D3DXCreateTextureFromFileInMemoryEx(g_pD3DDevice, PicAdr,822,D3DX_DEFAULT,D3DX_DEFAULT,1,0,D3DFMT_A8R8G8B8,D3DPOOL_MANAGED,D3DX_FILTER_NONE,D3DX_FILTER_NONE,D3DCOLOR_RGBA(0,0,0,255),NULL,NULL,&Bubble);
	DeleteObject(PicAdr);
	D3DXCreateSprite(g_pD3DDevice, &BubbleSprite);

	//Create the area query
UO(	if(FAILED(g_pD3DDevice->CreateQuery(D3DQUERYTYPE_OCCLUSION, &AreaQuery))) )
UO(		{DisplayError("Cannot Create Query Device, Your Graphics Card Does Not Support Pixel Shading, Only Supported By GF3/4 Ti/Fx or Radeon 9000+")} )

	//Create FPS Font
	#if FullCpu==1
		D3DXFONT_DESC Font;
		ZeroMemory(&Font, sizeof(D3DXFONT_DESC));
		strcpy(Font.FaceName, "Arial");
		Font.Height = -12;
		D3DXCreateFontIndirect(g_pD3DDevice, &Font, &g_pFont);
		g_pFont->OnResetDevice();
	#endif
	return true;
}

void ReleaseDX() //Releases Direct3D and devices
{	g_pD3DDevice->Release();
	SunTex->Release();
	Bubble->Release();
	BubbleSprite->Release();
	if(CloudTex!=NULL)
		CloudTex->Release();
	CloudTex=NULL;
	for(int i=0;i<NumberOfAreaRows*NumberOfAreaRows;i++)
	{
		Areas[i].VertBuf->Release();
UO(		Areas[i].BoxBuf->Release(); )
	}
	delete[] Areas;
	Areas=NULL;
	if(g_pFont)
		g_pFont->Release();
}

void MoveSun() //Move the sun and light position
{
	SunPos.x=(float)(cos(D3DXToRadian(SunAng))*40*sqrt(2)+5);
	SunPos.y=5;
	SunPos.z=(float)(sin(D3DXToRadian(SunAng))*40*sqrt(2)+5);

	D3DLIGHT9 light;
	ZeroMemory(&light, sizeof(D3DLIGHT9));
	light.Type       = D3DLIGHT_DIRECTIONAL;
	light.Diffuse.r  = 1.0f;
	light.Diffuse.g  = 1.0f;
	light.Diffuse.b  = 1.0f;
	light.Direction = D3DXVECTOR3(5,0,5)-SunPos; //Sun points directly towards point 5,0,5
	g_pD3DDevice->SetLight(0, &light);
}

void InitFractal() //Resets the fractal height map and rebuilds
{
	InInit=true; //Tells the engine not to create a vertex buffer yet
	if(FractalPointer!=NULL) //Clear any old height map
		delete[] FractalPointer;
	FractalPointer=new float[4]; //Make a new level 0 height map that is the plane y=0
	for(int i=0;i<4;i++)
		FractalPointer[i]=0;
	for(i=0;i<5;i++) //Move the fractal levels up to 5 (The maximum)
		FractalLevelInc();

	InInit=false; //Move the fractal to a vertex buffer
	MoveFractalToVB();

	//Set up other initial states
	MoveSun();
	MoveCamera();
}

void FractalLevelInc() //Increases the fractal height map array by 2 areas
{
	if(FractalLevel+1>MaxFracLevs)
		return;
	WriteProcessing();
	RemakeBuf=true;
	//Resize the height map array and fill with old values at new intervals
	int NumberOfOldRowCols=(1<<((FractalLevel++)*2))+1, NumberOfNewRowCols=(1<<(FractalLevel*2))+1;
	float *NewFractalPointer= new float[NumberOfNewRowCols*NumberOfNewRowCols];
	{
		int YRowCountOld=0,YRowCountNew=0;
		for(int y=0;y<NumberOfOldRowCols;y++,YRowCountOld+=NumberOfOldRowCols,YRowCountNew+=NumberOfNewRowCols*4)
			for(int x=0,XNew=0;x<NumberOfOldRowCols;XNew+=4)
				NewFractalPointer[XNew+YRowCountNew]=FractalPointer[(x++)+YRowCountOld];
	}
	delete[] FractalPointer;
	FractalPointer=NewFractalPointer;

	if(MulBy>.5) //Decrease the roughness size
		MulBy/=2;
	else
		MulBy/=8;
	FractalLevelMake();
}

void FractalLevelDec() //Decreases the fractal height map array by 2 areas
{
	if(FractalLevel-1<MinFracLevs)
		return;
	WriteProcessing();
	RemakeBuf=true;
	//Contract the height map array and fill with old values, will retain same values as before last increase
	int NumberOfOldRowCols=(1<<((FractalLevel-1)*2))+1, NumberOfNewRowCols=(1<<(FractalLevel*2))+1;
	float *NewFractalPointer= new float[NumberOfOldRowCols*NumberOfOldRowCols];
	{
		int YRowCountOld=0,YRowCountNew=0;
		for(int y=0;y<NumberOfOldRowCols;y++,YRowCountOld+=NumberOfOldRowCols,YRowCountNew+=NumberOfNewRowCols*4)
			for(int x=0,XNew=0;x<NumberOfOldRowCols;XNew+=4)
				NewFractalPointer[(x++)+YRowCountOld]=FractalPointer[XNew+YRowCountNew];
	}
	delete[] FractalPointer;
	FractalPointer=NewFractalPointer;
	FractalLevel--;

	if(MulBy>.4) //Increase the roughness size
		MulBy*=2;
	else
		MulBy*=8;
	MoveFractalToVB();
}

void FractalLevelMake() //Reinterpolates current fractal level height values
{
	WriteProcessing();
	if(++RemRand>5) //Check to see if in the example mode, and if not, seed the pseudo random number generator
		srand(GetTickCount());
	else
		srand(18781085);
	//Runs the fill height values loop twice, once for double spaces, once for single
	int NumberOfRowCols=(1<<(FractalLevel*2))+1,MoveRow2=NumberOfRowCols*2,DoubleAdd=4;
	for(int i=2;i>0;i--)
	{
		int YRow=0;
		for(int y=i;y<NumberOfRowCols;y+=DoubleAdd) //Fill in Center point formula with optimizations to remove imuls through pointers
		{
			YRow=y*NumberOfRowCols;
			for(int x=i;x<NumberOfRowCols;x+=DoubleAdd)
			{
				float Average=0;
				int XYStart=x+YRow;
				Average+=FractalPointer[XYStart-i-MoveRow2];
				Average+=FractalPointer[XYStart+i-MoveRow2];
				Average+=FractalPointer[XYStart-i+MoveRow2];
				Average+=FractalPointer[XYStart+i+MoveRow2];
				FractalPointer[XYStart]=Average/4+RandF()*MulBy;
			}
		}

		YRow=0; //Fill in border points with optimizations to remove imuls through pointers
		int FlipStart=0;
		for(y=0;y<NumberOfRowCols;y+=i)
		{
			YRow=y*NumberOfRowCols;
			int x;
			if(FlipStart==0)
			{
				FlipStart=1;
				x=i;
			}
			else
			{
				FlipStart=0;
				x=0;
			}
			for(;x<NumberOfRowCols;x+=DoubleAdd)
			{
				float Average=0;
				int XYStart=x+YRow;
				if(x==0) //Check to see if current height point is on an edge
				{
					Average+=FractalPointer[XYStart+i];
					Average+=FractalPointer[XYStart+i];
					Average+=FractalPointer[XYStart-MoveRow2];
					Average+=FractalPointer[XYStart+MoveRow2];
				}
				else if(x==NumberOfRowCols-1)
				{
					Average+=FractalPointer[XYStart-i];
					Average+=FractalPointer[XYStart-i];
					Average+=FractalPointer[XYStart-MoveRow2];
					Average+=FractalPointer[XYStart+MoveRow2];				
				}
				else if(y==0)
				{
					Average+=FractalPointer[XYStart-i];
					Average+=FractalPointer[XYStart+i];
					Average+=FractalPointer[XYStart+MoveRow2];
					Average+=FractalPointer[XYStart+MoveRow2];
				}
				else if(y==NumberOfRowCols-1)
				{
					Average+=FractalPointer[XYStart-i];
					Average+=FractalPointer[XYStart+i];
					Average+=FractalPointer[XYStart-MoveRow2];
					Average+=FractalPointer[XYStart-MoveRow2];
				}
				else //Not on an edge
				{
					Average+=FractalPointer[XYStart-i];
					Average+=FractalPointer[XYStart+i];
					Average+=FractalPointer[XYStart-MoveRow2];
					Average+=FractalPointer[XYStart+MoveRow2];
				}
				FractalPointer[XYStart]=Average/4+RandF()*MulBy;
			}
		}
		MoveRow2=NumberOfRowCols; //Decrement values for next height fill
		DoubleAdd=2;
	}

	//Removes jaggie protruding pieces from the final level
	if(FractalLevel==5)
	{
		float *CurrentPointer=FractalPointer;
		for(int y=0;y<NumberOfRowCols;y++)
			for(int x=0;x<NumberOfRowCols;x++,CurrentPointer++)
			{
				//Checks how many height points in cross around current height point are lower than .01
				while(true)
				{
					int AddUp=0;
					if(x!=NumberOfRowCols-1 && *CurrentPointer-.01f>*(CurrentPointer+1))
						AddUp++;
					if(x!=0 &&  *CurrentPointer-.01f>*(CurrentPointer-1))
						AddUp++;
					if(y!=NumberOfRowCols-1 && *CurrentPointer-.01f>*(CurrentPointer+NumberOfRowCols))
						AddUp++;
					if(y!=0 && *CurrentPointer-.01f>*(CurrentPointer-NumberOfRowCols))
						AddUp++;
					
					if(AddUp<3) //If 3 or more height points are lower, lower current by .01 and retry
						break;
					*CurrentPointer-=.01f;
				}
			}
	}
	MoveFractalToVB();
}

void MoveFractalToVB() //Moves the fractal height values to a vertex buffer
{
	if(InInit) //Still making more height values
		return;

	int NumberOfRowCols=1<<(FractalLevel*2);
	if(RemakeBuf) //The size of the height map array has changed
	{
		int NumberOfAreas=NumberOfAreaRows*NumberOfAreaRows;
		if(Areas!=NULL) //Delete old areas and vertex buffers
		{
			for(int i=0;i<NumberOfAreas;i++)
			{
				Areas[i].VertBuf->Release();
UO(				Areas[i].BoxBuf->Release(); )
			}
			delete[] Areas;
		}
		RemakeBuf=false;
		NumberOfAreaRows=2*(int)pow(4,FractalLevel-3);
		NumberOfAreas=NumberOfAreaRows*NumberOfAreaRows;
		
		//Make new areas, vertex buffers, and cloud texture
		Areas = new AreaStruct[NumberOfAreas];
		for(int i=0;i<NumberOfAreas;i++)
		{
			g_pD3DDevice->CreateVertexBuffer(2081*sizeof(Vertex), D3DUSAGE_WRITEONLY, D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_NORMAL,D3DPOOL_MANAGED, &Areas[i].VertBuf, NULL);
UO(			g_pD3DDevice->CreateVertexBuffer(36*sizeof(Vertex), D3DUSAGE_WRITEONLY, D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_NORMAL,D3DPOOL_MANAGED, &Areas[i].BoxBuf, NULL); )
		}
		if(CloudTex!=NULL)
			CloudTex->Release();
		CloudTex=NULL;

		if(CloudOn)
			D3DXCreateTexture(g_pD3DDevice,NumberOfRowCols,NumberOfRowCols,1,0,D3DFMT_A8R8G8B8,D3DPOOL_MANAGED,&CloudTex);
	}

	float SpanSize=40.0f/(NumberOfRowCols-1);
	Vertex *VertsRem = new Vertex[++NumberOfRowCols*NumberOfRowCols]; //Makes a width X height vertex array for height values
	//Fills in the positions and colors in the vertex array
	{
		float *FracAdr=FractalPointer;
		Vertex *Verts=VertsRem;
		float YLoc=0;
		for(int y=0;y<NumberOfRowCols;y++,YLoc+=SpanSize)
		{
			float XLoc=0;
			for(int x=0;x<NumberOfRowCols;x++,XLoc+=SpanSize,Verts++)
			{
				float NewYHgt=*(FracAdr++);

				Verts->x=XLoc;
				Verts->y=NewYHgt;
				Verts->z=YLoc;
				
				if(NewYHgt>1.2f)
				{
					DWORD ColLower=(int)((NewYHgt-1.2f)*55.5f)+128;
					Verts->Color=ColLower+(ColLower<<8)+(ColLower<<16)|OrOp;
				}
				else if(NewYHgt>0)
				{
					DWORD ColLower=(int)(NewYHgt*44.5f);
					Verts->Color=((ColLower+60)<<8)+((ColLower+90)<<16)|OrOp;

				}
				else if(NewYHgt<-1.2f)
					Verts->Color=(int)((NewYHgt+1.2f)*-55.5f)+128|OrOp;
				else
					Verts->Color=((int)(NewYHgt*-44.5f+128))<<8|OrOp;
			}
		}
	}

	//Gets face normals for each polygon
	D3DXVECTOR3 *FaceNormals = new D3DXVECTOR3[--NumberOfRowCols*NumberOfRowCols*2],*TFaceNormals=FaceNormals;
	{
		Vertex *TempVerts=VertsRem;
		for(int y=0;y<NumberOfRowCols;y++,TempVerts++)
			for(int x=0;x<NumberOfRowCols;x++,TempVerts++)
			{
				Vertex *VertsQuad[4]={TempVerts,TempVerts+NumberOfRowCols+1,TempVerts+1,TempVerts+NumberOfRowCols+2};

				if(VertsQuad[0]->y>-1.2f||VertsQuad[1]->y>-1.2f||VertsQuad[2]->y>1.2f||VertsQuad[3]->y>-1.2f)
				{
					D3DXVECTOR3 Vec1(VertsQuad[0]->x-VertsQuad[1]->x,VertsQuad[0]->y-VertsQuad[1]->y,VertsQuad[0]->z-VertsQuad[1]->z);
					D3DXVECTOR3 Vec2(VertsQuad[0]->x-VertsQuad[2]->x,VertsQuad[0]->y-VertsQuad[2]->y,VertsQuad[0]->z-VertsQuad[2]->z);
					D3DXVECTOR3 Vec3(VertsQuad[3]->x-VertsQuad[2]->x,VertsQuad[3]->y-VertsQuad[2]->y,VertsQuad[3]->z-VertsQuad[2]->z);
					D3DXVECTOR3 Vec4(VertsQuad[3]->x-VertsQuad[1]->x,VertsQuad[3]->y-VertsQuad[1]->y,VertsQuad[3]->z-VertsQuad[1]->z);
					
					D3DXVec3Normalize(TFaceNormals, D3DXVec3Cross(TFaceNormals, &Vec1, &Vec2));
					TFaceNormals++;
					D3DXVec3Normalize(TFaceNormals, D3DXVec3Cross(TFaceNormals, &Vec3, &Vec4));
					TFaceNormals++;
				}
				else
				{
					*(TFaceNormals++)=D3DXVECTOR3(0,0,0);
					*(TFaceNormals++)=D3DXVECTOR3(0,0,0);
				}

			}
	}
	
	//Fills in vertex normal information by interpolating between each face it touches
	{
		int NumRowM1D=(NumberOfRowCols++)*2;
		Vertex *RemVerts=(Vertex*)((BYTE*)VertsRem+sizeof(float)*3);
		for(int y=0,YRow=0;y<NumberOfRowCols;y++,YRow+=NumRowM1D)
		{
			for(int x=0,X2=0;x<NumberOfRowCols;x++,RemVerts++,X2+=2)
			{
				D3DXVECTOR3 *NormalVec=(D3DXVECTOR3*)RemVerts;
				if(x==0)
				{
					if(y==0)
						*NormalVec=FaceNormals[0];
					else
					{
						int a=YRow-NumRowM1D;
						*NormalVec=FaceNormals[a];
						*NormalVec+=FaceNormals[a+1];
						if(y!=NumberOfRowCols-1)
							*NormalVec+=FaceNormals[YRow];
					}
				}
				else if(x==NumberOfRowCols-1)
				{
					if(y==NumberOfRowCols-1)
						*NormalVec=FaceNormals[YRow-1];
					else
					{
						int a=YRow+NumRowM1D-1;
						*NormalVec+=FaceNormals[a];
						*NormalVec+=FaceNormals[a-1];
						if(y!=0)
							*NormalVec=FaceNormals[YRow-1];
					}
				}
				else if(y==0 || y==NumberOfRowCols-1)
				{
					D3DXVECTOR3 *A;
					if(y!=0)
						A=&FaceNormals[YRow-NumRowM1D-2+X2];
					else
						A=&FaceNormals[X2-1];
					*NormalVec=*A;
					*NormalVec+=*(A+1);
					*NormalVec+=*(A+2);
				}
				else
				{
					D3DXVECTOR3 *A=&FaceNormals[X2+YRow];
					*NormalVec=*A;
					*NormalVec+=*(A-1);
					*NormalVec+=*(A-2);
					A-=NumRowM1D;
					*NormalVec=*A;
					*NormalVec+=*(A+1);
					*NormalVec+=*(A-1);
				}
				D3DXVec3Normalize(NormalVec,NormalVec); //Normalize the vector for better lighting
			}
		}
	}
	delete[] FaceNormals;
	NumberOfRowCols--;

	//Fill in each area with vertex buffer info and bounding box info
	int Interval=(int)(NumberOfRowCols/NumberOfAreaRows);
	float YL=0,AddArea=Interval*SpanSize;
	for(int y1=0,VertBufOn=0;y1<NumberOfRowCols;y1+=Interval,YL+=AddArea)
	{
		float XL=0;
		for(int x1=0;x1<NumberOfRowCols;x1+=Interval,VertBufOn++,XL+=AddArea)
		{
			//Fill In vertecies, uses pointers to remove imuls
			Vertex *Verts=NULL;
			Vertex *VertsRemA=&VertsRem[x1+y1*(NumberOfRowCols+1)];
			Areas[VertBufOn].VertBuf->Lock(0, 0, (void**)&Verts, 0);
			Vertex *NewVertsRem=Verts;

			*(Verts++)=*VertsRemA;
			*(Verts++)=*(VertsRemA+NumberOfRowCols+1);
			VertsRemA++;
	
			for(int y=y1;y<y1+Interval;y++)
			{
				for(int x=x1+1;x<x1+Interval+1;x++,VertsRemA++)
				{
					*(Verts++)=*VertsRemA;
					*(Verts++)=*(VertsRemA+NumberOfRowCols+1);
				}

				VertsRemA+=NumberOfRowCols;
				*(Verts++)=*(VertsRemA+NumberOfRowCols+1);
				VertsRemA--;
				y++;

				for(x=x1+Interval-1;x>x1-1;x--,VertsRemA--)
				{
					*(Verts++)=*VertsRemA;
					*(Verts++)=*(VertsRemA+NumberOfRowCols+1);
				}
				
				if(y+1==y1+Interval)
					break;
				VertsRemA+=NumberOfRowCols+2;
				*(Verts++)=*(VertsRemA+NumberOfRowCols+1);
				VertsRemA++;
			}

			//Find the max and min height values for the bounding box
			float MinY=1000,MaxY=-1000;
			for(int i=0;i<2079+2;i++)
			{
				float YVal=NewVertsRem[i].y;
				if(YVal<MinY)
					MinY=YVal;
				if(YVal>MaxY)
					MaxY=YVal;
			}

			Areas[VertBufOn].VertBuf->Unlock();
			
			//Fills bounding box point and plane info
			AreaStruct *FillArea=&Areas[VertBufOn];
			FillArea->PointBounds[0].x=FillArea->PointBounds[2].x=FillArea->PointBounds[4].x=FillArea->PointBounds[6].x=XL;
			FillArea->PointBounds[0].y=FillArea->PointBounds[1].y=FillArea->PointBounds[4].y=FillArea->PointBounds[5].y=MinY;
			FillArea->PointBounds[0].z=FillArea->PointBounds[1].z=FillArea->PointBounds[2].z=FillArea->PointBounds[3].z=YL;
			FillArea->PointBounds[1].x=FillArea->PointBounds[3].x=FillArea->PointBounds[5].x=FillArea->PointBounds[7].x=XL+AddArea;
			FillArea->PointBounds[2].y=FillArea->PointBounds[3].y=FillArea->PointBounds[6].y=FillArea->PointBounds[7].y=MaxY;
			FillArea->PointBounds[4].z=FillArea->PointBounds[5].z=FillArea->PointBounds[6].z=FillArea->PointBounds[7].z=YL+AddArea;
			D3DXPlaneFromPoints(&FillArea->PlaneBounds[0], &FillArea->PointBounds[0], &FillArea->PointBounds[1], &FillArea->PointBounds[2]);
			D3DXPlaneFromPoints(&FillArea->PlaneBounds[1], &FillArea->PointBounds[6], &FillArea->PointBounds[7], &FillArea->PointBounds[5]);
			D3DXPlaneFromPoints(&FillArea->PlaneBounds[2], &FillArea->PointBounds[2], &FillArea->PointBounds[6], &FillArea->PointBounds[4]);
			D3DXPlaneFromPoints(&FillArea->PlaneBounds[3], &FillArea->PointBounds[7], &FillArea->PointBounds[3], &FillArea->PointBounds[5]);
			D3DXPlaneFromPoints(&FillArea->PlaneBounds[4], &FillArea->PointBounds[2], &FillArea->PointBounds[3], &FillArea->PointBounds[6]);
			D3DXPlaneFromPoints(&FillArea->PlaneBounds[5], &FillArea->PointBounds[1], &FillArea->PointBounds[0], &FillArea->PointBounds[4]);
#if UseOcclusion==1
			static char MapVertecies[24]={0,4,2,6, 1,3,5,7, 0,1,4,5, 2,6,3,7, 0,2,1,3, 4,5,6,7};
			Vertex *NewVertecies;
			Areas[VertBufOn].BoxBuf->Lock(0, 0, (void**)&NewVertecies, 0);
			Vertex  *TempVertecies=NewVertecies;
			for(i=0;i<24;i+=4,TempVertecies+=6)
			{
				*(D3DXVECTOR3*)&TempVertecies[0]=FillArea->PointBounds[MapVertecies[i+0]];
				*(D3DXVECTOR3*)&TempVertecies[1]=FillArea->PointBounds[MapVertecies[i+1]];
				*(D3DXVECTOR3*)&TempVertecies[2]=FillArea->PointBounds[MapVertecies[i+2]];

				*(D3DXVECTOR3*)&TempVertecies[3]=FillArea->PointBounds[MapVertecies[i+3]];
				*(D3DXVECTOR3*)&TempVertecies[4]=FillArea->PointBounds[MapVertecies[i+2]];
				*(D3DXVECTOR3*)&TempVertecies[5]=FillArea->PointBounds[MapVertecies[i+1]];

				for(int p=0;p<6;p++)
				{
					*(D3DXVECTOR3*)&TempVertecies[p].xn=D3DXVECTOR3(0,0,0);
					TempVertecies[p].Color|=0x01000001; //change to 770000ff
				}
			}	
			Areas[VertBufOn].BoxBuf->Unlock();
#endif
		}
	}
	delete[] VertsRem;
	
	//Extra stuff that needs to be done on a VB remake
	if(CloudOn)
		MakeCloud();

	MoveCamera();
	NonFullCPU_DoFrame();
}

void MakeCloud() //Uses the height map to create a cloud texture
{
	int NumberOfRowCols=1<<(FractalLevel*2);
	D3DLOCKED_RECT LockedRect={NumberOfRowCols*4,NULL};
	CloudTex->LockRect(0,&LockedRect,NULL,D3DLOCK_DISCARD);
	DWORD *PutColor=(DWORD*)LockedRect.pBits;
	float *FracPntOn=FractalPointer;
	for(int y=0;y<NumberOfRowCols;y++,FracPntOn++)
		for(int x=0;x<NumberOfRowCols;x++,FracPntOn++,PutColor++)
		{
			int NewColor=(int)((*FracPntOn)*64);
			*PutColor=0x00FFFFFF;
			if(NewColor<0)
				NewColor=-NewColor;
			NewColor+=128;
			if(NewColor<200)
				*PutColor=NewColor|(NewColor<<8)|(NewColor<<16)|0xCC000000;
		}
	CloudTex->UnlockRect(0);
}

void DoFrame() //Renders a frame
{
	g_pD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, WorldAt[2]>-1.2f ? D3DCOLOR_RGBA(255,255,255,255) : D3DCOLOR_RGBA(0,0,255,255), 1.0f, 0);
	g_pD3DDevice->BeginScene();

UO(	g_pD3DDevice->SetRenderState(D3DRS_ZWRITEENABLE, false); )
UO(	g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE); )
UO(	/*g_pD3DDevice->SetRenderState(D3DRS_COLORWRITEENABLE, D3DCOLORWRITEENABLE_ALPHA|D3DCOLORWRITEENABLE_BLUE|D3DCOLORWRITEENABLE_GREEN|D3DCOLORWRITEENABLE_RED);*/ )

	//Render areas
	AreaStruct* AreaOn=Areas;
	for(int y=0;y<NumberOfAreaRows;y++)
		for(int x=0;x<NumberOfAreaRows;x++,AreaOn++)
			if(QuadInFrustrum(AreaOn)) //Only render an area if it is viewable within the viewing frustrum
			{
#if UseOcclusion==0
				g_pD3DDevice->SetStreamSource(0, AreaOn->VertBuf, 0, sizeof(Vertex));
				g_pD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2079);
#else
				DWORD PixelsDrawn;
				AreaQuery->Issue(D3DISSUE_BEGIN);
				g_pD3DDevice->SetStreamSource(0, AreaOn->BoxBuf,0, sizeof(Vertex));
				g_pD3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 12);
				AreaQuery->Issue(D3DISSUE_END);
				while (AreaQuery->GetData((void*)&PixelsDrawn, sizeof(DWORD), D3DGETDATA_FLUSH) == S_FALSE);
				if(PixelsDrawn!=0)
				{
					g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
					g_pD3DDevice->SetRenderState(D3DRS_ZWRITEENABLE, true);
					g_pD3DDevice->SetStreamSource(0, AreaOn->VertBuf, 0, sizeof(Vertex));
					g_pD3DDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2079);
					g_pD3DDevice->SetRenderState(D3DRS_ZWRITEENABLE, false);
					g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
				}
#endif
			}

NUO(g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE); ) //Turn on alpha blending for clouds and water
UO(	g_pD3DDevice->SetRenderState(D3DRS_ZWRITEENABLE, true); )
UO(	g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW); )


	bool SetTexNull=false; //Variable to see if new texture has been set

	//Draws clouds
	if(CloudOn)
	{
		static float CloudVerts[20] = {80,4.5,80,0,0, -40,4.5,80,0,12, 80,4.5,-40,12,0, -40,4.5,-40,12,12};
		static float CloudMove[9]={.3f,0,.2f,0,.6f,0,.1f,0,.8f};
		g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, false);
		if(!FogOn)
			g_pD3DDevice->SetRenderState(D3DRS_RANGEFOGENABLE, false);
		g_pD3DDevice->SetFVF(D3DFVF_XYZ | D3DFVF_TEX1);
		g_pD3DDevice->SetTexture(0, CloudTex);
		SetTexNull=true;
		
		//Draws 5 clouds with alpha to create a layered effect
		for(int i=0;i<9;i+=2)
		{
			if(CloudMove[i]>12)
				CloudMove[i]=0;
			//Moves the clouds texture, so as to not have to move the clouds themselves
			CloudVerts[3]=CloudVerts[4]=CloudVerts[8]=CloudVerts[14]=CloudMove[i];
			CloudVerts[9]=CloudVerts[13]=CloudVerts[18]=CloudVerts[19]=8+CloudMove[i];
			g_pD3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, CloudVerts, 20);
		}
		if(!FogOn)
			g_pD3DDevice->SetRenderState(D3DRS_RANGEFOGENABLE, true);
		//Moves each cloud layer at a different rate
		CloudMove[0]+=.2f/20;
		CloudMove[2]+=.1f/20;
		CloudMove[4]+=.075f/20;
		CloudMove[6]+=.06f/20;
		CloudMove[8]+=.08f/20;
	}

	//Draw the sun
	if(LightOn)
	{
		if(WorldAt[2]>-1.2)
			g_pD3DDevice->SetRenderState(D3DRS_FOGENABLE, false);
		if(!SetTexNull) //Only needs to perform operations if texture hasnt already been set from cloud
		{
			SetTexNull=true;
			g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, false);
			g_pD3DDevice->SetFVF(D3DFVF_XYZ | D3DFVF_TEX1);
		}
		//Billboard sun towards viewer and set at SunPos
		D3DXVECTOR3 x(MatView._11*1.5f, MatView._21*1.5f, MatView._31*1.5f), ym(MatView._12*1.5f, MatView._22*1.5f, MatView._32*1.5f);
		D3DXVECTOR3 CC[4]={-x+ym, x+ym, -x-ym, x-ym};
		float SunVerts[20]={CC[0].x+SunPos.x+WorldAt[0],CC[0].y+SunPos.y,CC[0].z+SunPos.z+WorldAt[1],0,0,  CC[1].x+SunPos.x+WorldAt[0],CC[1].y+SunPos.y,CC[1].z+SunPos.z+WorldAt[1],1,0,  CC[2].x+SunPos.x+WorldAt[0],CC[2].y+SunPos.y,CC[2].z+SunPos.z+WorldAt[1],0,1,  CC[3].x+SunPos.x+WorldAt[0],CC[3].y+SunPos.y,CC[3].z+SunPos.z+WorldAt[1],1,1};
		g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &SunProj);
		g_pD3DDevice->SetTexture(0, SunTex);
		g_pD3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, SunVerts, 20);
		g_pD3DDevice->SetTransform(D3DTS_PROJECTION, &MatProj);		
	}
	else
		g_pD3DDevice->SetRenderState(D3DRS_FOGENABLE, false);
	if(SetTexNull) //Resets texture and FVF if a texture has been used
	{
		g_pD3DDevice->SetTexture(0, NULL);
		g_pD3DDevice->SetFVF(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_NORMAL);
	}

	//Draws alpha blended water line
	Vertex Verts[4] = { {WorldAt[0]-20,-1.2f,WorldAt[1]-20,0,1,0,0x990000FF},{WorldAt[0]-20,-1.2f,WorldAt[1]+20,0,1,0,0x990000FF},{WorldAt[0]+20,-1.2f,WorldAt[0]-20,0,1,0,0x990000FF},{WorldAt[0]+20,-1.2f,WorldAt[0]+20,0,1,0,0x990000FF}};
	g_pD3DDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Verts, sizeof(Vertex));
	g_pD3DDevice->SetRenderState(D3DRS_FOGENABLE, true);

	 //If under water, draw bubbles
	if(FogOn)
	{
		BubbleSprite->Begin(D3DXSPRITE_ALPHABLEND);
		for(int i=0;i<20;i++)
		{
			BubbleSprite->Draw(Bubble, NULL, NULL, &Bubbles[i].Current,0xFFFFFFFF);
			if(Bubbles[i].FrameOn++>Bubbles[i].Speed)  //If bubble has hit a new frame
			{
				Bubbles[i].FrameOn=0;
				Bubbles[i].Current.y-=Bubbles[i].Amplitude;
				if(Bubbles[i].Current.y>-8) //If bubble is on screen, find through how many amplitudes it has gone, get bubble displacement from its modulus and 20, and multiply by magnitude
					Bubbles[i].Current.x=(ScreenWidth/2-8)+Bubbles[i].Magnitude*BubbleDisplacementArray[(((ScreenHeight/2-8)-(int)Bubbles[i].Current.y)/Bubbles[i].Amplitude)%20];
				else //If bubble is not on screen, reset to center of screen
					Bubbles[i].Current=D3DXVECTOR3(ScreenWidth/2-8,ScreenHeight/2-8,0);
			}
		}
		BubbleSprite->End();
	}

	//Resets variables for next frame
NUO(g_pD3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, false); )
	if(LightOn) g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, true);
	#if FullCpu==1 //Draws fps
		FPSDraw();
	#endif
	g_pD3DDevice->EndScene();
	g_pD3DDevice->Present(NULL, NULL, NULL, NULL);
	FrameOn++;
}

void MoveCamera() //Moves the camera when an action is performed
{
	//Check to make sure the user hasnt gone past the bounds of the map
	if(WorldAt[0]<0)
		WorldAt[0]=0;
	if(WorldAt[0]>40)
		WorldAt[0]=40;
	if(WorldAt[1]<0)
		WorldAt[1]=0;
	if(WorldAt[1]>40)
		WorldAt[1]=40;
	//Checks to make sure the user hasnt gone past the camera Y-bounds
	if(CameraAngle2<D3DX_PI/-2.0f)
		CameraAngle2=D3DX_PI/-2.0f;
	if(CameraAngle2>D3DX_PI/4.0f)
		CameraAngle2=D3DX_PI/4.0f;

	//Gets the new camera rotation matrix according to the CameraAngle variables
	D3DXMATRIXA16 MatViewAngle,MatViewMove;
	D3DXMatrixRotationYawPitchRoll(&MatViewAngle,CameraAngle,(float)cos(CameraAngle)*CameraAngle2,(float)sin(CameraAngle)*CameraAngle2);
	int NumberOfRowsCols=(1<<(FractalLevel*2))+1;

	//Determines what area the eye is over
	float SpanSize=40.0f/(NumberOfRowsCols-1),RelativeX=WorldAt[0],RelativeY=WorldAt[1];
	while(RelativeX>SpanSize)
		RelativeX-=SpanSize;
	while(RelativeY>SpanSize)
		RelativeY-=SpanSize;

	float StartX=WorldAt[0]-RelativeX, StartY=WorldAt[1]-RelativeY;
	Vertex Vert1,Vert2,Vert3;
	int StartAdr=(int)(StartX/SpanSize)+(int)(StartY/SpanSize)*NumberOfRowsCols;
	//Determines which triangle of the area the eye is over
	if(SpanSize-RelativeY>RelativeX)
	{
		Vert1.x=StartX;
		Vert1.y=FractalPointer[StartAdr];
		Vert1.z=StartY;
		Vert2.x=StartX;
		Vert2.y=FractalPointer[StartAdr+NumberOfRowsCols];
		Vert2.z=StartY+SpanSize;
		Vert3.x=StartX+SpanSize;
		Vert3.y=FractalPointer[StartAdr+1];
		Vert3.z=StartY;
	}
	else
	{
		Vert1.x=StartX+SpanSize;
		Vert1.y=FractalPointer[StartAdr+1+NumberOfRowsCols];
		Vert1.z=StartY+SpanSize;
		Vert2.x=StartX+SpanSize;
		Vert2.y=FractalPointer[StartAdr+1];
		Vert2.z=StartY;
		Vert3.x=StartX;
		Vert3.y=FractalPointer[StartAdr+NumberOfRowsCols];
		Vert3.z=StartY+SpanSize;
	}
	//Finds the normal of the triangle the eye is over
	D3DXVECTOR3 Ray1=*(D3DXVECTOR3*)&Vert1-*(D3DXVECTOR3*)&Vert2,Ray2=*(D3DXVECTOR3*)&Vert1-*(D3DXVECTOR3*)&Vert3,NormalVec;
	D3DXVec3Cross(&NormalVec,&Ray1,&Ray2);
	//Determines the new Y position based on the normal of the triangle the eye is over
	float NewY=((Vert1.x-WorldAt[0])*NormalVec.x+(Vert1.z-WorldAt[1])*NormalVec.z)/NormalVec.y+Vert1.y+.2f;
	WorldAt[2]=NewY;

	if(NewY<-1.2f)
	{
		if(!FogOn)  //If the eye just moved under water, set new states
		{
			g_pD3DDevice->SetRenderState(D3DRS_RANGEFOGENABLE, false);
			g_pD3DDevice->SetRenderState(D3DRS_FOGSTART, F2DW(.1f));
			g_pD3DDevice->SetRenderState(D3DRS_FOGEND, F2DW(.4f));
			g_pD3DDevice->SetRenderState(D3DRS_FOGDENSITY, F2DW(.1f));
			
			//Set up initial bubble variables
			srand(GetTickCount());
			for(int i=0;i<20;i++)
			{
				Bubbles[i].Current=D3DXVECTOR3(ScreenWidth/2-8,ScreenHeight/2-8,0); //Start in exact center of screen
				Bubbles[i].Magnitude=(RandF()+1)*30; //0 - 30
				Bubbles[i].Amplitude=rand()%6+3; //3-8
				Bubbles[i].Speed=rand()%4; //0-3
				Bubbles[i].FrameOn=Bubbles[i].Speed-i; //Start each bubble 1 after the other, starting with the first
			}
		}
		FogOn=true;
		g_pD3DDevice->SetRenderState(D3DRS_FOGCOLOR, (int)((NewY+1.2)*-90)+128); //Change color of fog depending on how deep under water
	}
	else if(FogOn) //If the eye just moved above water, set new states
	{
		g_pD3DDevice->SetRenderState(D3DRS_RANGEFOGENABLE, true);
		g_pD3DDevice->SetRenderState(D3DRS_FOGSTART, F2DW(1.0f));
		g_pD3DDevice->SetRenderState(D3DRS_FOGEND, F2DW(30.0f));
		g_pD3DDevice->SetRenderState(D3DRS_FOGDENSITY, F2DW(.5f));
		g_pD3DDevice->SetRenderState(D3DRS_FOGCOLOR, 0xFFFFFF);
		FogOn=false;
	}

	//Multiples the camera angle matrix and eye translation matrix to get the new Camera view
	D3DXMatrixTranslation(&MatViewMove,-WorldAt[0],-NewY,-WorldAt[1]);
	D3DXMatrixMultiply(&MatView,&MatViewMove,&MatViewAngle);
	g_pD3DDevice->SetTransform(D3DTS_VIEW, &MatView);

	//Inverse the multiply of the view and projection matrix to get the viewing frustrum bounds, much faster than recalculating each areas bounds every time the camera moves
	D3DXMATRIXA16 mat;
	D3DXMatrixInverse(&mat, NULL, D3DXMatrixMultiply(&mat, &MatView, &MatProj));
	for(int i=0; i<8; i++)
	{
		D3DXVECTOR3 FrustPoint((i&1)?1.0f:-1.0f,(i&2)?1.0f:-1.0f,(i&4)?1.0f:0);
		D3DXVec3TransformCoord(&CameraPoints[i], &FrustPoint,&mat);
	}
	D3DXPlaneFromPoints(&CameraPlanes[0], &CameraPoints[0], &CameraPoints[1], &CameraPoints[2]);
	D3DXPlaneFromPoints(&CameraPlanes[1], &CameraPoints[6], &CameraPoints[7], &CameraPoints[5]);
	D3DXPlaneFromPoints(&CameraPlanes[2], &CameraPoints[2], &CameraPoints[6], &CameraPoints[4]);
	D3DXPlaneFromPoints(&CameraPlanes[3], &CameraPoints[7], &CameraPoints[3], &CameraPoints[5]);
	D3DXPlaneFromPoints(&CameraPlanes[4], &CameraPoints[2], &CameraPoints[3], &CameraPoints[6]);
	D3DXPlaneFromPoints(&CameraPlanes[5], &CameraPoints[1],	 &CameraPoints[0], &CameraPoints[4]);
	NonFullCPU_DoFrame();
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) //Processes events
{
	switch (msg)
	{
		case WM_PAINT:  //If something causes a window to be invalidated, repaint without having to redraw scene
			g_pD3DDevice->Present(NULL, NULL, NULL, NULL);
			break;
	#if FullCpu==0
		case WM_MOUSEMOVE: //If using windows messages instead of DirectInput, move camera angles according to mouse
			CameraAngle+=D3DX_PI/360*(MousePos[0]-LOWORD(lParam));
			CameraAngle2+=D3DX_PI/360*(MousePos[1]-HIWORD(lParam));
			MousePos[0]=LOWORD(lParam);
			MousePos[1]=HIWORD(lParam);			
			MoveCamera();
			return 0;
		case WM_CHAR: //If using windows messages instead of DirectInput, process keys
			ProcessKey((char)wParam, hWnd);
			return 0;
	#endif
		case WM_DESTROY: //Close the program
			PostQuitMessage(0);
			return 0;
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

bool QuadInFrustrum(AreaStruct* AreaOn) //Checks to see if a quadtree area is within the viewing frustrum
{
	BYTE bOutside[8];
	ZeroMemory(&bOutside, sizeof(bOutside));
	
	//Checks first to see if any one point is inside the frustrum
	for(int i=0;i<8;i++)
	{
		for(int p=0;p<6;p++)
		if(CameraPlanes[p].a * AreaOn->PointBounds[i].x + CameraPlanes[p].b * AreaOn->PointBounds[i].y + CameraPlanes[p].c * AreaOn->PointBounds[i].z + CameraPlanes[p].d < 0)
			bOutside[i] |= (1 << p);
		if(bOutside[i] == 0)
			return true;
	}
	
	//Checks to see if all points are on the outside of any 1 plane using values from previous loop
	if((bOutside[0] & bOutside[1] & bOutside[2] & bOutside[3] & bOutside[4] & bOutside[5] & bOutside[6] & bOutside[7])!= 0)
		return false;
	
	//Sees if any frustrum edges intersect the plane of a quad area (partially inside)
	D3DXVECTOR3 *VecPointer = (D3DXVECTOR3*)&AreaOn->PointBounds;
	D3DXPLANE *PlnPointer = (D3DXPLANE*)&AreaOn->PlaneBounds;
	D3DXVECTOR3 edge[12][2] = {CameraPoints[0], CameraPoints[1], CameraPoints[2], CameraPoints[3], CameraPoints[0], CameraPoints[2], CameraPoints[1], CameraPoints[3], CameraPoints[4], CameraPoints[5], CameraPoints[6], CameraPoints[7], CameraPoints[4], CameraPoints[6], CameraPoints[5], CameraPoints[7], CameraPoints[0], CameraPoints[4], CameraPoints[2], CameraPoints[6], CameraPoints[1], CameraPoints[5], CameraPoints[3], CameraPoints[7]};
	D3DXVECTOR3 face[6][4] ={VecPointer[0], VecPointer[2], VecPointer[3], VecPointer[1],VecPointer[4], VecPointer[5], VecPointer[7], VecPointer[6],VecPointer[0], VecPointer[4], VecPointer[6], VecPointer[2],VecPointer[1], VecPointer[3], VecPointer[7], VecPointer[5],VecPointer[2], VecPointer[6], VecPointer[7], VecPointer[3],VecPointer[0], VecPointer[4], VecPointer[5], VecPointer[1],};
	D3DXVECTOR3 *pEdge, *pFace;
	pEdge = &edge[0][0];
	for(i=0; i<12; i++,pEdge+=2)
	{
		pFace = &face[0][0];
		for(int p=0; p<6; p++,pFace+=4)
		{
			if(EdgeIntersectsFace(pEdge, pFace, &PlnPointer[p]))
				return true;
		}
	}

	//Sees if any quad area edges intersect the plane of the frustrum (partially inside)
	D3DXVECTOR3 edge2[12][2] = {VecPointer[0], VecPointer[1],VecPointer[2], VecPointer[3],VecPointer[0], VecPointer[2],VecPointer[1], VecPointer[3],VecPointer[4], VecPointer[5],VecPointer[6], VecPointer[7],VecPointer[4], VecPointer[6],VecPointer[5], VecPointer[7],VecPointer[0], VecPointer[4],VecPointer[2], VecPointer[6],VecPointer[1], VecPointer[5],VecPointer[3], VecPointer[7],};
	D3DXVECTOR3 face2[6][4] ={CameraPoints[0], CameraPoints[2], CameraPoints[3], CameraPoints[1],CameraPoints[4], CameraPoints[5], CameraPoints[7], CameraPoints[6],CameraPoints[0], CameraPoints[4], CameraPoints[6], CameraPoints[2],CameraPoints[1], CameraPoints[3], CameraPoints[7], CameraPoints[5],CameraPoints[2], CameraPoints[6], CameraPoints[7], CameraPoints[3],CameraPoints[0], CameraPoints[4], CameraPoints[5], CameraPoints[1],};
	pEdge = &edge2[0][0];
	for(i=0;i<12;i++)
	{
		pFace = &face2[0][0];
		for(int p=0;p<6;p++)
		{
			if(EdgeIntersectsFace(pEdge, pFace, &CameraPlanes[p]))
				return true;
			pFace += 4;
		}
		pEdge += 2;
	}

	//Sees if the frustrum is contained in the bounding box
	for(i=0; i<6; i++)
		if(PlnPointer[i].a * CameraPoints[0].x + PlnPointer[i].b * CameraPoints[0].y + PlnPointer[i].c * CameraPoints[0].z + PlnPointer[i].d<0)
			return false;
	return true;
}

bool EdgeIntersectsFace(D3DXVECTOR3* pEdges, D3DXVECTOR3* pFacePoints, D3DXPLANE* pPlane) //Checks to see if an edge intersects a face, only used in the previous function
{
	float fDist1 = pPlane->a * pEdges[0].x + pPlane->b * pEdges[0].y + pPlane->c * pEdges[0].z + pPlane->d, fDist2 = pPlane->a * pEdges[1].x + pPlane->b * pEdges[1].y + pPlane->c * pEdges[1].z + pPlane->d;
	if(fDist1 > 0 && fDist2 > 0 || fDist1 < 0 && fDist2 < 0)
		return false;
	D3DXVECTOR3 ptIntersection;
	if(D3DXPlaneIntersectLine(&ptIntersection, pPlane, &pEdges[0], &pEdges[1])==NULL)
		return false;
	FLOAT fAbsA = (pPlane->a > 0 ? pPlane->a : -pPlane->a), fAbsB = (pPlane->b > 0 ? pPlane->b : -pPlane->b), fAbsC = (pPlane->c > 0 ? pPlane->c : -pPlane->c);
	D3DXVECTOR2 facePoints[4], point;
	if(fAbsA > fAbsB && fAbsA > fAbsC)
	{
		for(int i=0;i<4;i++)
		{
			facePoints[i].x = pFacePoints[i].y;
			facePoints[i].y = pFacePoints[i].z;
		}
		point.x = ptIntersection.y;
		point.y = ptIntersection.z;
	}
	else if(fAbsB > fAbsA && fAbsB > fAbsC)
	{
		for(int i=0;i<4;i++)
		{
			facePoints[i].x = pFacePoints[i].x;
			facePoints[i].y = pFacePoints[i].z;
		}
		point.x = ptIntersection.x;
		point.y = ptIntersection.z;
	}
	else
	{
		for(int i=0;i<4;i++)
		{
			facePoints[i].x = pFacePoints[i].x;
			facePoints[i].y = pFacePoints[i].y;
		}
		point.x = ptIntersection.x;
		point.y = ptIntersection.y;
	}
	float x0=facePoints[0].x, x1=facePoints[1].x, x2=facePoints[2].x, y0=facePoints[0].y, y1=facePoints[1].y, y2=facePoints[2].y;
	bool bClockwise = false;
	if(x1*y2-y1*x2-x0*y2+y0*x2+x0*y1-y0*x1<0)
		bClockwise = true;
	x2 = point.x;
	y2 = point.y;
	for(int i=0;i<4;i++)
	{
		x0 = facePoints[i].x;
		y0 = facePoints[i].y;
		if(i<3)
		{
			x1 = facePoints[i+1].x;
			y1 = facePoints[i+1].y;
		}
		else
		{
			x1 = facePoints[0].x;
			y1 = facePoints[0].y;
		}
		if((x1*y2-y1*x2-x0*y2+y0*x2+x0*y1-y0*x1>0)==bClockwise)
			return false;
	}
	return true;
}

void WriteProcessing() //Writes the word "Processing" on the screen
{
	HDC MainDC=GetDC(MainHwnd);
	SetTextColor(MainDC, 0xFF);
	SetBkMode(MainDC, TRANSPARENT);
	RECT DrawRect={0,ScreenHeight/2-10,ScreenWidth,ScreenHeight/2+10};
	DrawText(MainDC, "Processing", 10, &DrawRect, DT_CENTER|DT_VCENTER);
	ReleaseDC(MainHwnd, MainDC);
}

void ProcessKey(char key, HWND hWnd) //Processes keyboard input
{
	#if FullCpu==1 //If running with DirectInput, remap VirtualKeys to ascii
		static char RemapKeys[51]={0,27,'1','2','3','4','5','6','7','8','9','0','-','=',9,0,'q','w','e','r','t','y','u','i','o','p','[',']',13,0,'a','s','d','f','g','h','j','k','l',';','\'','`',0,0,'z','x','c','v','b','n','m'};
		key=RemapKeys[key];
	#endif
	static DWORD KeyPressed[5]={0,0,0,0,0};
	switch(key)
	{
		case '=':
		case '+': //Increase the fractal level
			if(KeyPressed[0]+2==FrameOn) //Only do so once when the key goes down
			{
				KeyPressed[0]=FrameOn++;
				break;
			}
			KeyPressed[0]=FrameOn;
			FractalLevelInc();
			break;
		case '-': //Decrease the fractal level
			if(KeyPressed[1]+2==FrameOn) //Only do so once when the key goes down
			{
				KeyPressed[1]=FrameOn++;
				break;
			}
			KeyPressed[1]=FrameOn;

			FractalLevelDec();
			break;
		case 'r':FractalLevelMake();break; //Remake the current fractal level
		case 13: //Remake a completly new fractal set
			FractalLevel=0;
			CameraAngle=CameraAngle2=0;
			MulBy=4;
			InitFractal();
			break;
		case 'w': //Move foward
			WorldAt[0]+=(float)sin(CameraAngle)*-MoveSpeed;
			WorldAt[1]+=(float)cos(CameraAngle)*MoveSpeed;
			MoveCamera();
			break;
		case 's': //Move backward
			WorldAt[0]+=(float)sin(CameraAngle)*MoveSpeed/3;
			WorldAt[1]+=(float)cos(CameraAngle)*MoveSpeed/-3;
			MoveCamera();
			break;
		case 'a': //Move left
			WorldAt[0]+=(float)cos(CameraAngle)*MoveSpeed/-3;
			WorldAt[1]+=(float)sin(CameraAngle)*MoveSpeed/-3;
			MoveCamera();
			break;
		case 'd': //Move right
			WorldAt[0]+=(float)cos(CameraAngle)*MoveSpeed/3;
			WorldAt[1]+=(float)sin(CameraAngle)*MoveSpeed/3;
			MoveCamera();
			break;
		case 'l': //Toggle on/off the light
			if(KeyPressed[2]+2==FrameOn) //Only do so once when the key goes down
			{
				KeyPressed[2]=FrameOn++;
				break;
			}
			KeyPressed[2]=FrameOn;
			LightOn=!LightOn;
			g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, LightOn);
			NonFullCPU_DoFrame();
			break;
		case 'm': //Recenter the mouse cursor
			SetCursorPos(ScreenWidth/2, ScreenHeight/2);
			MousePos[0]=ScreenWidth/2-4;
			MousePos[1]=ScreenHeight/2-23;
			break;
		case 'e': //Goto the example fractal
			RemRand=0;
			#if FullCpu==0
				SendMessage(hWnd, WM_CHAR, 13, 0);
			#else
				ProcessKey(28,hWnd);
			#endif
			break;
		case 27: //Quit the program
			PostQuitMessage(0);
			break;
		case 'f': //Tottle on/off fullscreen
			ReleaseDX();
			d3dpp.Windowed =!d3dpp.Windowed;
			FogOn=!FogOn;
			RemakeBuf=true;
			InitDX();
			MoveFractalToVB();
			break;
		case 'j': //Move the sun counterclockwise
			SunAng+=5;
			if(SunAng==360)
				SunAng=0;
			MoveSun();
			NonFullCPU_DoFrame();
			break;
		case 'k': //Move the sun clockwise
			SunAng-=5;
			if(SunAng==-1)
				SunAng=359;
			MoveSun();
			NonFullCPU_DoFrame();
			break;
		case 'c': //Toggle on/off the clouds
			if(KeyPressed[3]+2==FrameOn) //Only do so once when the key goes down
			{
				KeyPressed[3]=FrameOn++;
				break;
			}
			KeyPressed[3]=FrameOn;
			WriteProcessing();
			CloudOn=!CloudOn;
			if(CloudOn)
			{
				int NumberOfRowCols=1<<(FractalLevel*2);
				D3DXCreateTexture(g_pD3DDevice,NumberOfRowCols,NumberOfRowCols,1,0,D3DFMT_A8R8G8B8,D3DPOOL_MANAGED,&CloudTex);
				MakeCloud();
			}
			else
			{
				CloudTex->Release();
				CloudTex=NULL;
			}
			NonFullCPU_DoFrame();
			break;
		case 'o': //Toggle on/off wireframe
			if(KeyPressed[4]+2==FrameOn) //Only do so once when the key goes down
			{
				KeyPressed[4]=FrameOn++;
				break;
			}
			KeyPressed[4]=FrameOn;
			static int WireFrame=D3DFILL_SOLID;
			WireFrame^=(D3DFILL_SOLID^D3DFILL_WIREFRAME);
			g_pD3DDevice->SetRenderState(D3DRS_FILLMODE, WireFrame);
			NonFullCPU_DoFrame();
			break;
	}
}

#if FullCpu==1
	void FPSDraw() //Draws the fps on the screen, only needed if full cpu is being used
	{
		static RECT DrawRect={0,0,100,100};
		static DWORD StoreTicks, OldTicks, TotalFrames,LastTotalFrames;
		char PrintFPS[9];
		//Print the average frames from the last 2 seconds onto the screen
		FillMemory(PrintFPS,8,32);
		PrintFPS[8]=0;
		DWORD CurrentTicks=GetTickCount();
		DWORD AverageFPS=(DWORD)(1000.0f*TotalFrames/(float)(CurrentTicks-OldTicks));
		ltoa(AverageFPS, PrintFPS, 10);
		PrintFPS[5]='f';PrintFPS[6]='p';PrintFPS[7]='s';
		for(int i=1;i<5;i++)
			if(PrintFPS[i]==0)
				PrintFPS[i]=32;
		g_pFont->DrawText(NULL, PrintFPS, 8, &DrawRect, DT_LEFT, 0xFFFF0000);
		TotalFrames++;
		if((CurrentTicks-OldTicks)>2000) //After 2 seconds is up, move the number frames from the last second only into a buffer to keep the framerate accurate
		{
			TotalFrames-=LastTotalFrames;
			LastTotalFrames=TotalFrames;
			OldTicks=StoreTicks;
			StoreTicks=CurrentTicks;
		}
	}
#endif